// -*- Mode: vala; indent-tabs-mode: nil; tab-width: 4 -*-
/***
  BEGIN LICENSE

  Copyright (C) 2012-2013 Mario Guerriero <mario@elementaryos.org>
  This program is free software: you can redistribute it and/or modify it
  under the terms of the GNU Lesser General Public License version 3, as
  published    by the Free Software Foundation.

  This program is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranties of
  MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
  PURPOSE.  See the GNU General Public License for more details.

  You should have received a copy of the GNU General Public License along
  with this program.  If not, see <http://www.gnu.org/licenses>

  END LICENSE
***/

using Gtk;
using AppCenter.Widgets;

namespace AppCenter.Pages {
    public class UpdatesTree : TreeView {
        // Signals
        public signal void update_added (TreeIter iter);
        
        // Vars
        private IconTheme theme;
        private new ListStore model;
        private TreeIter iter;
        private new TreeSelection selection;
        
        private GLib.List<AppStore.ModelApp> apps = new GLib.List<AppStore.ModelApp> ();
        
        public UpdatesTree () {
			theme = IconTheme.get_default ();
			theme.append_search_path ("/usr/share/app-install/icons");
			
			Type t_string = typeof (string);
            model = new ListStore (3,
                t_string, // Icon name
                t_string, // Name and description
                typeof (AppStore.ModelApp)  // id
            );
            
            set_model (model);
            
            selection = get_selection ();
            selection.mode = Gtk.SelectionMode.SINGLE;
            
            CellRendererPixbuf icon_cell = new CellRendererPixbuf ();
            icon_cell.height = 56;
            icon_cell.width = 56;
            icon_cell.stock_size = IconSize.DIALOG;
            
            CellRendererText text = new CellRendererText ();
            text.ellipsize = Pango.EllipsizeMode.END;
            
            insert_column_with_attributes (-1, "Icon", icon_cell, "icon-name", 0);
            insert_column_with_attributes (-1, "Name", text, "markup", 1);
            
            get_column (1).set_expand (true);
            get_column (1).clicked ();
            
            rules_hint = true;
            
            selection.changed.connect (on_selection_changed);

            headers_visible = false;
        }
        
        public void append_app (AppStore.ModelApp app) {            
            
            // Do not show duplicates
            bool is_duplicate = false;
            apps.foreach ((a) => {
                if (a.id == app.id)
                    is_duplicate = true;
            });
            apps.append (app);
            
            if (!is_duplicate) {
                // Do it better!
                while (Gtk.events_pending()) {
                    Gtk.main_iteration();
                }
                
                model.append (out iter);
                model.set (iter, 0, app.icon, 1, "<b>"+Utils.escape_text (app.name)+"</b>\n"+Utils.escape_text (app.description), 2, app); 
                
                update_added (iter);
            }
        }
        
        public void clear () {
            // FIXME: it is not cool to use that kind of hackish solutions
            selection.changed.disconnect (on_selection_changed);
            model.clear();
            selection.changed.connect (on_selection_changed);
        }
        
        public bool is_empty () {
            Gtk.TreeIter iter;
            return !model.get_iter_first (out iter);
        }
        
        public void on_selection_changed () {
            // Getting objects...
            /*Gtk.TreeIter? iter = null;
            Gtk.TreeModel? mod = null;
            selection.get_selected (out mod, out iter);
            var path = mod.get_path (iter);
            // Show selection
            get_cursor(out this.path, null);
            if (path != null) {
                AppStore.ModelApp val;
                model.get_iter(out iter, this.path);
                model.get(iter, 2, out val);
                activated_row(val);
            }
            // Clean selection
            selection.unselect_all ();*/
        }
       
    }
    
    public class UpdatesInfoBar : InfoBar {
        
        // Signals
        public signal void update_start ();
        public signal void update_end ();
        
        // Widgets
        private Label text;
        private Button install_button;
        private ProgressBar progress_bar;
       
        public UpdatesInfoBar () {
            message_type = MessageType.INFO;
            ((Box) get_action_area ()).orientation = Orientation.HORIZONTAL;
            Box main = get_content_area () as Box;
            main.orientation = Orientation.HORIZONTAL;
            main.spacing = 2;
            orientation = Orientation.HORIZONTAL;
        
            text = new Label (("Updating..."));
            text.use_markup = true;
            text.ellipsize = Pango.EllipsizeMode.END;
            main.pack_start (text, false, false, 0);
            
            install_button = new Button.with_label (_("Install updates"));
            install_button.clicked.connect (() => {
                update_start ();
                // Set up widgets
                install_button.set_visible (false);
                text.no_show_all = false;
                progress_bar.no_show_all = false;
                text.set_visible (true);
                progress_bar.set_visible (true);
                // Install
                // TODO: not yet implemented
                progress_bar.set_fraction (0.8);
            });
            main.pack_end (install_button, false, false, 0);
            //add_button (_("Install updates"), ResponseType.INSTALL);
            
            progress_bar = new ProgressBar ();
            progress_bar.pulse_step = 0.1;
            main.pack_end (progress_bar, false, false, 0);
            
            text.no_show_all = true;
            progress_bar.no_show_all = true;
        }
    
    }
    
    public class UpdatesView : BlankBox {
        
        private UpdatesTree updates_view;
        
        public UpdatesView () {
            base (Orientation.VERTICAL, 0, 0);
            
            // InfoBar
            var infobar = new UpdatesInfoBar ();
            infobar.update_start.connect (() => {
                updates_view.set_sensitive (false);
            });
            infobar.update_end.connect (() => {
                updates_view.set_sensitive (true);
            });
            
            pack_start (infobar, false, true, 0);
            
            // Updates tree view widget
            updates_view = new UpdatesTree ();
            ScrolledWindow updates_view_scroll = new ScrolledWindow (null, null);
            updates_view_scroll.set_policy (PolicyType.NEVER, PolicyType.AUTOMATIC);
            updates_view_scroll.add (updates_view);
            updates_view_scroll.scroll_child.connect (() => {
                return false;
            });
            
            pack_start (updates_view_scroll, true, true, 0);
            
            // Loading spinner
            var spinner = new Spinner ();
            spinner.set_size_request (32, 32);
            
            pack_start (spinner, true, false, 0);
            
            // Apps manager
            apps_manager.loading_started.connect ((type) => {
                if (type == AppStore.LoadingType.UPDATES) {
                    spinner.start ();
                    spinner.set_visible (true);
                    infobar.set_visible (false);
                    updates_view_scroll.set_visible (false);
                }
            });  
            apps_manager.loading_finished.connect ((type) => {
                if (type == AppStore.LoadingType.UPDATES) {
                    spinner.stop ();
                    spinner.set_visible (false);
                    infobar.set_visible (true);
                    updates_view_scroll.set_visible (true);
                }
            });            
                        
        }
        
        public void update_received (AppStore.ModelApp app) {
            updates_view.append_app (app);
        }
        
    }
}
